import sltxpkg.globals
import sltxpkg.config
import sltxpkg.dep
import sltxpkg.generate
import sltxpkg.util
import sltxpkg.commands
import sltxpkg.prompt
import sltxpkg.command_config
import sltxpkg.cmd_line_args
import sltxpkg.log_control



import sltxpkg.lithie.compile.latexmk_mos as lmos
from sltxpkg.globals import LOGGER

# Configs for latexmk


def tool_glossary(recipe):
    LOGGER.info("Tool: Glossary")
    lmos.append_local2global_config('glossary')


def tool_index(recipe):
    LOGGER.info("Tool: index")
    lmos.append_local2global_config('index')



import glob
import os
import random
import shutil
import tempfile
from os.path import abspath, basename, splitext
from pathlib import Path

import sltxpkg.config as sc
import sltxpkg.data.recipes
import sltxpkg.globals as sg
import sltxpkg.lithie.compile.cooking.recipe_exceptions as rex
import sltxpkg.lithie.compile.tools as tools
import sltxpkg.util as su
import yaml
from importlib_resources import files
from sltxpkg.globals import print_idx
from sltxpkg.util import create_multiple_replacer

# TODO: in case of error pack the directory with tar and ship it to the folder
BRACE_REPLACER = create_multiple_replacer({
    ':braceo:': '{',
    ':bracee:': '}',
})


class Recipe():

    settings = {
        'name': '',
        'author': '',
        'hooks': {
                'pre': [],
                'in': [],  # before retrieval
            'post': []  # before cleanup
        },
        # cleanup will be determined by global settings
        'cleanup_cmds': [],
        'args': '',
        'extra_args': [],
        'tools': [],
        # to retrieve files:
        'wanted_files': [],
        'executable': '',
        'run': [],
        'quiet': ''
    }

    quiet = ''

    def __init__(self, recipe_path: str, file: str, idx: int):
        super().__init__()
        self.file = file
        self.idx = idx
        recipe_full_path = recipe_path
        if not os.path.isfile(recipe_full_path):
            recipe_full_path = str(
                files(sltxpkg.data.recipes).joinpath(recipe_path))
        if not os.path.isfile(recipe_full_path):
            print_idx(self.idx, "Recipe " + recipe_full_path +
                      " was not found. Exiting.")
            exit(1)
        print_idx(self.idx, "Loading recipe: " + recipe_full_path)
        y_conf = su.load_yaml(recipe_full_path)
        self.settings = {**self.settings, **y_conf}
        if sg.args.quiet:
            self.quiet = self.settings['quiet']
        self.__process_tools()
        self.__sanitize_extra_args()  # we need them as a single string

    def __process_tools(self):
        for tool in self.settings['tools']:
            getattr(tools, 'tool_' + tool)(self)

    def __sanitize_extra_args(self):
        self.settings['extra_args'].extend(sg.args.extra_arguments)
        self.settings['extra_args'] = " ".join(self.settings['extra_args'])

    @staticmethod
    def get_default_recipes() -> [str]:
        return [f for f in os.listdir() if f.endswith(".recipe")]

    # format; TODO: maybe cache the results
    def __f(self, t: str) -> str:
        for _ in range(sg.configuration[sg.C_FORMAT_MAX]):
            t = t.format(**self.settings, **sg.configuration, file=self.file,
                         filenoext=splitext(self.file)[0], do_quiet=self.quiet,
                         file_base_noext=splitext(basename(self.file))[0],
                         tmp=tempfile.gettempdir(),
                         out_dir=os.path.join("{cache_dir}", su.sanitize_filename(abspath(self.file))))
        return BRACE_REPLACER(t)

    def __runcmds(self, cmds: [str]):
        for cmd in cmds:
            cmd = self.__f(cmd)  # expand
            if sg.args.verbose:
                print_idx(self.idx, "  - " + cmd)
            os.system(cmd)

    def __runhooks(self, hookid: str):
        if sg.args.verbose:
            print_idx(self.idx, "> Hooks for \"" + hookid + "\"")
        self.__runcmds(self.settings['hooks'][hookid])

    def __critical_abort(self, code: int):
        print_idx(self.idx, "Collecting files in working directory")
        archive = shutil.make_archive(os.path.join(
            os.getcwd(), 'sltx-log-' + su.get_now() + '-' + su.sanitize_filename(self.file)), 'zip', self.__f("{out_dir}"))
        print_idx(self.idx, "  - Created: \"" + archive +
                  "\" (" + os.path.basename(archive) + ")")

        # We have to force latexmk into think it has to re-run
        # TODO: We should check if not aux but we do this
        # We do this as we need a change and the percent itself gets consumed
        # This ensures a different sequence every time that will be deleted
        # on a successful run
        with open(self.__f("{out_dir}/{file_base_noext}.aux"), 'a') as f:
            f.write('%% sltx errormark' + str(random.random()) +
                    " - " + str(random.random()))
        # automatic analyze
        os.system('sltx analyze "' + archive + '"')
        raise rex.RecipeException(archive,
                                  'Recipe for '+str(self.idx)+' failed with code: '+str(code)+'. See logfile: \"' + archive + "\"")

    def __save_files(self, our_dir: str):
        # Get the resulting file(s)
        print_idx(self.idx, "> Retrieving resulting files to \"" + our_dir + "\"")
        got_files = []
        for wf in self.settings['wanted_files'] + sg.configuration[sg.C_WANTED_FILES]:
            wf = self.__f(wf)
            if sg.args.verbose:
                print_idx(
                    self.idx, "  - Retrieving files for pattern \"" + wf + "\"")
            wanted = glob.glob(os.path.join(
                sg.configuration[sg.C_WORKING_DIR], wf))
            for f in wanted:
                if sg.args.verbose:
                    print_idx(self.idx, "Saving \"" + f + "\" ")
                shutil.copy2(f, our_dir)
            got_files += wanted

        if not sg.args.verbose:
            print_idx(self.idx, "Saved files (" + str(got_files) + ")")

    def run(self):
        """Executes the configured Recipe
        """
        print_idx(self.idx, "Processing file: " + self.file, pre='\n')
        sc.assure_dirs()  # Ensure Working diSr and texmf home
        sc.assure_dir('file cache', self.__f("{out_dir}"), True)
        print_idx(self.idx, self.__f(
            "> Running recipe \"{name}\" by \"{author}\"."))
        self.__runhooks('pre')

        print_idx(self.idx, "> Running the compile commands (" +
                  str(len(self.settings['run'])) + ")")
        for i, cmd in enumerate(self.settings['run']):
            cmd = self.__f(cmd)  # expand
            if sg.args.verbose:
                print_idx(self.idx, "  - " + cmd)
            fback = os.system(cmd)
            if fback != 0:
                print_idx(self.idx,
                          "\033[31m  ! The command failed. Initiating critical abort...\033[m")
                self.__critical_abort(fback)

        self.__runhooks('in')
        self.__save_files(os.getcwd())
        self.__runhooks('post')
        if sg.configuration[sg.C_CLEANUP]:
            print_idx(self.idx, "> Cleaning up (configured by configuration)")
            self.__runcmds(self.settings['cleanup_cmds'])



import os
from sys import platform

import sltxpkg.data.latexmk.configs
import sltxpkg.util as su
from importlib_resources import as_file, files
from sltxpkg.globals import LOGGER

# We have to configure latexmk configuration-files os-dependent
# NOTE: no cygwin support


def get_root() -> str:
    """Return the root folder of the current platform

    Returns:
        str: root folder of the current platform
    """
    if platform == "linux":
        return os.path.expandvars("$HOME/.latexmkrc")
    elif platform == "darwin":
        return os.path.expandvars("$HOME/.latexmkrc")
    elif platform == "win32":
        return "C:\latexmk\LatexMk"


def get_config(name: str) -> str:
    """Get local mkrc configuration

    Args:
        name (str): The required configuration file name.

    Returns:
        str: The content of the requested file.
    """
    return files(sltxpkg.data.latexmk.configs).joinpath(name + '.mkrc').read_text()


def append_local2global_config(name: str) -> None:
    """Appends an included configuration to the system configuration

    Args:
        name (str): The requirested configuration
    """
    __append_to_global_config(get_config(name), name)


def __append_to_global_config(text: str, guard: str) -> None:
    file = get_root()
    START_GUARD = "# sltx START " + guard
    mode = 'a'

    if os.path.isfile(file) and su.file_contains(file, START_GUARD):
        LOGGER.info(
            "Global latexmk config already contains config for %s. Skipping.", guard)
        return

    with open(file, mode) as f:
        f.write("\n" + START_GUARD + "\n")
        f.writelines(text)
        f.write("# sltx END " + guard + "\n")



def add_argument_to_parser(p, *args, **kwargs):
    p.add_argument(*args, **kwargs)


def add_parser(p, name, arguments, **kwargs):
    np = p.add_parser(name, **kwargs)
    for args in arguments:
        add_argument_to_parser(np, *args.args, **args.kwargs)


class Arg:
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs


class Commands:
    def __init__(self, cmds: dict):
        self.cmds = cmds

    def helper_values(self):
        return map(lambda x: x[0], self.cmds.values())

    def generate(self, p):
        for command in self.cmds.items():
            aliases = Commands.__alias_from_cmd(command)
            add_parser(p, command[0], command[1][2], **((dict(command[1][1].kwargs,
                                                              aliases=aliases)) if len(aliases) != 0 else command[1][1].kwargs))

    @staticmethod
    def __alias_from_cmd(alias: tuple) -> list:
        return alias[1][0][1]

    def __cmd_from_name(self, tc: str):
        return self.cmds[tc][0][0]

    def __find_in_alias(self, tc: str):
        matching_commands = [
            alias[0]
            for alias in self.cmds.items()
            if tc in Commands.__alias_from_cmd(alias)]
        # if multiple we take the first (should not happen)
        return self.__cmd_from_name(matching_commands[0])

    def get_cmd(self, search_key: str):
        return self.__cmd_from_name(search_key) if search_key in self.cmds else self.__find_in_alias(search_key)



import shutil
import tempfile
import glob
import os.path
import re

ANALYZER_PATTERN = re.compile(
    '^! |Error:|Undefined control sequence|improper alph|Incomplete \\\\if|Runaway preamble\\?|has an extra|Permission denied|not loadable: Metric|Extra alignment tab has been|Can\'t create output|too long|Runaway argument|al parameter number|Misplaced (alignment)?|doesn\'t match|Invalid UTF-8|forgotten \\\\end|ERROR|Missing')
ARCHIVE_PATTERN = re.compile('.*(\.tar(\.gz)?$|\.zip|\.7z)')

ANALYZE_DIVIDER = "\033[38;5;247m    # -------------------------------------\033[m"


class Analyzer():
    def __init__(self, file: str, idx: int):
        super().__init__()
        self.file = file
        self.idx = idx

    def __unpack_or_cp(self) -> str:
        if(re.match(ARCHIVE_PATTERN, self.file)):
            print("\033[38;5;31m" + self.file.replace(os.getcwd(),
                                                      '\033[38;5;247m[cwd]\033[38;5;31m') + "\033[m")
            target = tempfile.TemporaryDirectory(prefix='sltx-ua-log').name
            shutil.unpack_archive(self.file, extract_dir=target)
            return target
        else:
            print("Analyze Logfile...")
            return ""

    def __analyze_file(self, analyze_f: str):
        print("\033[38;5;31m  - "+analyze_f+"\033[m", end=' ')
        self.fine = True
        with open(analyze_f) as tl:
            self.offset_ptr = 0
            for line in tl.readlines():
                self.__analyze_line(line)
        if self.fine:
            print('\033[32m(fine)\033[m')

    def __analyze_line(self, line):
        if re.search(ANALYZER_PATTERN, line):
            self.offset_ptr = 6
            if self.fine:
                print()
                self.fine = False
            print('\033[38;5;64m', end='')
        if self.offset_ptr > 0:
            self.__print_error_line(line)

    def __print_error_line(self, line):
        print('    >', line, end='\033[m')
        self.offset_ptr -= 1
        if not self.offset_ptr:
            print(ANALYZE_DIVIDER)

    def __analyze_folder(self, target: str):
        templogs = []
        for ext in ('*.log', '*.ilg', '*.glg', '*.alg', '*.blg', '*.sltx-log'):
            templogs.extend(glob.glob(os.path.join(target, ext)))
        for templog in templogs:
            self.__analyze_file(templog)

    # TODO: make this more sensitive (makeindex logs etc...)
    def analyze(self):
        target: str = self.__unpack_or_cp()
        if target:
            self.__analyze_folder(target)
        else:
            self.__analyze_file(self.file)
        print()



import logging
import logging.config
import re
from datetime import datetime

from sltxpkg.util import create_multiple_replacer

LOG_STR_LONG = '%(asctime)s [%(levelname)-8s@%(filename)-10s;%(lineno)-4d] %(message)s'
LOG_STR = '%(message)s'
LOG_STR = '%(level_color)s%(message)s%(color_reset)s'

RESET_SEQ = "\033[0m"
ENCAPSULE_SEQ = "\033[%dm%s" + RESET_SEQ

COLORS = {
    'WARNING': '\033[33m',
    'INFO': '',
    'DEBUG': '\033[35m',
    'CRITICAL': '\033[95m',
    'ERROR': '\033[31m'
}


__log__replacer = create_multiple_replacer({
    'True': ENCAPSULE_SEQ % (92, 'True'),
    'False': ENCAPSULE_SEQ % (91, 'False')
})


def log_color_format(message: str) -> str:
    return __log__replacer(message)


class LithieColoredStreamFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color

    def format(self, record):

        if self.use_color and record.levelname in COLORS:
            record.level_color = COLORS[record.levelname]
        else:
            record.level_color = ""
        record.color_reset = RESET_SEQ
        msg = logging.Formatter.format(self, record)
        return log_color_format(msg)


class LithieLogFileHandler(logging.FileHandler):
    """Just like the default logger, but it suppresses a line break if the msg starts with \\u1
    """

    def emit(self, record) -> None:
        if record.msg and record.msg.startswith('\u0001'):
            record.msg = record.msg[1:]
            self.terminator = ''
        else:
            self.terminator = '\n'

        return super().emit(record)


class LithieLogStreamHandler(logging.StreamHandler):
    """Just like the default logger, but it suppresses a line break if the msg starts with \\u1
    """

    def emit(self, record) -> None:
        if record.msg and record.msg.startswith('\u0001'):
            record.msg = record.msg[1:]
            self.terminator = ''
        else:
            self.terminator = '\n'
        return super().emit(record)


# We keep only one logger as a beginning
logging.basicConfig(format=LOG_STR, datefmt='%Y-%m-%d %H:%M:%S')


def log_set_file_handler():
    """Update the file handler of the main 'LOGGER' to mirror outputs to a file
    """
    sltx_log_file_handler = LithieLogFileHandler(
        'sltx-{:%Y-%m-%d-%H-%M-%S-%f}.sltx-log'.format(datetime.now()))
    formatter = logging.Formatter(LOG_STR)
    sltx_log_file_handler.setFormatter(formatter)
    LOGGER.addHandler(sltx_log_file_handler)


LOGGER = logging.getLogger('sltx')
LOGGER.handlers.clear()
LOGGER.propagate = False
sltx_log_stream_handler = LithieLogStreamHandler()
# sltx_log_stream_handler.setFormatter(LithieColoredStreamFormatter(LOG_STR))
LOGGER.addHandler(sltx_log_stream_handler)
LOGGER.setLevel(logging.DEBUG)



import os
import sys

import yaml

from sltxpkg import globals as sg
from sltxpkg import util as su
from sltxpkg.globals import (C_AUTODETECT_DRIVERS, C_CACHE_DIR, C_CLEANUP,
                             C_CREATE_DIRS, C_DOWNLOAD_DIR, C_DRIVER_LOG,
                             C_DRIVER_PATTERNS, C_DRIVERS, C_TEX_HOME,
                             C_WORKING_DIR, DEFAULT_CONFIG)
from sltxpkg.log_control import LOGGER


def write_to_log(data: str):
    if sg.configuration[C_DRIVER_LOG].strip():
        with open(sg.configuration[C_DRIVER_LOG], 'a') as f:
            f.write(data)
            if not data.endswith('\n'):
                f.write("\n")


def load_configuration(file: str):
    """Apply given configuration file to the sltx config

    Args:
        file (str): The configuration file to load
    """
    y_conf = su.load_yaml(file)
    sg.configuration = {**sg.configuration, **y_conf}


def load_dependencies_config(file: str, target: dict) -> dict:
    """Apply given dependency file to the sltx dep list

    Args:
        file (str): The file to load
        target (dict): The target dependency-collection to append it to (won't be modified)

    Returns:
        dict: The target dict with the added dependencies
    """
    y_dep = su.load_yaml(file)
    return {**target, **y_dep}


def assure_dir(name: str, target_path: str, create: bool):
    if not os.path.isdir(target_path):
        if create:
            LOGGER.info("> %s: %s not found. Creating...", name, target_path)
            os.makedirs(target_path)
        else:
            LOGGER.error("! Not allowed to create " + name + ". Exit")
            sys.exit(1)


def assure_dirs():
    sg.configuration[C_TEX_HOME] = su.get_sltx_tex_home()  # expansion
    create = sg.configuration[C_CREATE_DIRS]
    assure_dir('Tex-Home', sg.configuration[C_TEX_HOME], create)

    for config, name in [(C_WORKING_DIR, 'Working-Dir'), (C_DOWNLOAD_DIR, 'Download-Dir'),
                         (C_CACHE_DIR, 'Cache-Dir')]:
        sg.configuration[config] = os.path.expanduser(
            sg.configuration[config])  # expansion
        assure_dir(name, sg.configuration[config], create)



# cookes the recipe :D
import sys
from concurrent import futures

import sltxpkg.globals as sg
import sltxpkg.lithie.compile.cooking.recipe_exceptions as rex
from sltxpkg.globals import LOGGER
from sltxpkg.lithie.compile.cooking.recipe import Recipe


def cook():
    """This will take the desired files and cooks the recipes accordingly.
    """
    recipe_path = sg.configuration[sg.C_DEFAULT_RECIPE] if sg.args.recipe is None else sg.args.recipe

    with futures.ThreadPoolExecutor(max_workers=sg.args.threads) as pool:
        runners = []
        for i, file in enumerate(sg.args.files):
            recipe = Recipe(recipe_path, file, i)
            runners.append((pool.submit(recipe.run), i, file))
        futures.wait([r[0] for r in runners])
        try:
            for runner, i, file in runners:
                if runner.result() is not None:
                    LOGGER.info(
                        "Status for runner: %d operating on file: %s", i, file)
                    LOGGER.info(runner.result())
        except rex.RecipeException as ex:
            print("\n\033[31m ! Processing of", file,
                  "failed for:", repr(ex), "\033[m")
            sys.exit(128)
        else:
            LOGGER.info("\n=Compiled all documents successfully=")



import sltxpkg.prompt as prompt
from sltxpkg.lithie.docker_mg import DockerCtrl
import sltxpkg.globals as sg
import sltxpkg.config as sc

# TODO: get valid names


def install():
    valid_profiles = ['tx-small', 'tx-default', 'tx-full']
    print("Please enter the profile you want. Valid names are:", valid_profiles)
    target = prompt.get(
        "Profile [{default}]", default=sg.configuration[sg.C_DOCKER_PROFILE]).lower()
    # TODO: do not exit, try anyway
    if target not in valid_profiles:
        sg.LOGGER.error("%s was not found in %s exiting for now",
                        valid_profiles, target)
        exit(1)

    _install(target)


def _install(target: str):
    docker_ctrl = DockerCtrl()
    docker_ctrl.update_img(target)

# TODO: clarify time in docker container as it is utc it must be the same as host!!!!
# TODO: keep container running to avoid full recompilations each time? (this should be the optimum!!)
# TODO: optimal would be to mount the working dir to the real world outside working dir to have it detached but saved
# TODO: in this case cleaning would not be needed!


def compile():
    sc.assure_dirs()
    docker_ctrl = DockerCtrl()
    profile = sg.configuration[sg.C_DOCKER_PROFILE] if sg.args.profile is None else sg.args.profile
    sltx_command = "sltx -t " + str(sg.args.threads) + " "

    if sg.args.log:
        sltx_command += "--log "

    if sg.args.verbose:
        sltx_command += "--verbose: "

    if sg.args.quiet:
        sltx_command += "--quiet "

    if sg.args.config is not None:
        sltx_command += "--config \"" + sg.args.config + "\" "

    sltx_command += "raw-compile "

    if sg.args.recipe is not None:
        sltx_command += "--recipe \"" + sg.args.recipe + "\" "

    if sg.args.extra_arguments is not None:
        sltx_command += "--args=\"" + " ".join(sg.args.extra_arguments) + "\" "

    for dep in sg.args.extra_dependencies:
        # will extend the dict with 'new' ones
        # should work even better if sltx-source.yaml files are present in the targets
        sltx_command += "--dependency \"" + dep + "\" "

    sltx_command += " ".join(['"' + f + '"' for f in sg.args.files])

    sg.LOGGER.info("Running command in docker: " + sltx_command)
    docker_ctrl.run_in_container(sg.args.dock_as_root, profile, sltx_command)



import argparse
import os

import sltxpkg.util as su
from sltxpkg.command_config import Arg, Commands
from sltxpkg.commands import (cmd_analyze_logfile, cmd_auto_setup, cmd_cleanse,
                              cmd_compile, cmd_dependency, cmd_docker,
                              cmd_gen_gha, cmd_raw_compile, cmd_version)


def valid_file(arg: str) -> str:
    if arg is None or arg.strip() == "":
        raise ValueError("arg vas none or empty")
    if not os.path.isfile(arg):
        raise FileNotFoundError("\"" + arg + "\" must be an existing file")
    return arg


file_help = "the file(s) to load; they will be processed."
file_tx = "file.tex"

sub_parser = Commands({
    'analyze': ((cmd_analyze_logfile, ['log']),
                Arg(description='Analyze a logfile for errors.'),
                [
                    Arg('files', metavar='log.zip',
                        type=str, nargs='*', help=file_help)]
                ),
    'dependency': ((cmd_dependency, ['dep']),
                   Arg(description='Install dependencies on the host system.'),
                   [
        Arg('-l', '--local', metavar='path', dest='local_path', default=None,
            help="This will install the dependency file into the given directory. This might be useful if the compilation should be handled by an online editor. Use '.' to use the current directory."),
        Arg('deps', metavar='dep.yml', type=valid_file, nargs='+',
            help="the file(s) to load the dependencies from.")
    ]),
    'docker': ((cmd_docker, ['do']), Arg(description='Manage the containers to compile with sltx.'), []),
    'compile': ((cmd_compile, ['cmp']), Arg(description="Compile documents with previously installed containers. If docker was disabled this will default to the same behavior as \"raw-compile\" passing on the recipe."),
                [
        Arg('-p', '--profile', dest='profile',
                help="allows to override the configured docker profile. This will enable docker automatically."),
        Arg('-r', '--recipe', dest='recipe',
            help='The recipe to instruct the main compile routine.', required=False, default=None),
        Arg('--root', dest='dock_as_root', action='store_true',
            help='Run the docker container with the lithie-root setup. This may lead to permission errors when you want to delete the caches.', required=False),
        Arg('-l', '--local-texmf', dest='local_texmf', action='store_true', default=False,
            help='include the local texmf tree in the docker container', required=False),
        Arg('--no-local-texmf', dest='local_texmf', action='store_false',
            help='exclude the local texmf tree in the docker container', required=False),
        Arg('-d', '--dependencies', dest='extra_dependencies', action='append', default=[],
            help='additional dependency files to download before the installation. May be supplied multiple times.', required=False),
        Arg('-a', '--args', action='append', metavar='ARGUMENT(S)', dest='extra_arguments', default=[],
            help="Extra arguments. Make sure to prepend them with the appropriate '-' or '--', they will be used as extra_arguments in the recipe."),
        Arg('files', metavar=file_tx, type=str, nargs='*',
            help=file_help)
    ]),
    'raw-compile': ((cmd_raw_compile, ['raw-cmp']), Arg(description="Compile documents using a recipe. This will not start any docker container but will be executed inside one as well."),
                    [
        Arg('-r', '--recipe', dest='recipe',
            help='The recipe to instruct the main compile routine.', required=False, default=None),
        Arg('files', metavar=file_tx, type=str, nargs='*',
            help=file_help),
        Arg('-a', '--args', action='append', metavar='ARGUMENT(S)', dest='extra_arguments', default=[],
            help="Extra arguments. Make sure to prepend them with the appropriate '-' or '--', they will be used as extra_arguments in the recipe."),
        Arg('-d', '--dependency', dest='extra_dependencies', action='append', default=[],
            help='additional dependency files to download before the installation. May be supplied multiple times.', required=False)
    ]),
    'gen-gha': ((cmd_gen_gha, ['gha']), Arg(description='Generate a GitHub workflow To automate compilation.'), []),
    'cleanse': ((cmd_cleanse, ['cls']), Arg(description="This will clean all additional sltx-files in the current directory (like \"sltx-log-*\" files). It may clean more, if you pass the corresponding flags. Please note, that cleanse will only read the current config. If you've changed some configurations they will be used."),
                [
        Arg('-C', '--cache', dest='cleanse_cache', action='store_true',
                help="If set, sltx will clean the cache."),
        Arg('--all', dest='cleanse_all', action='store_true',
            help="If set, sltx will clean the texmf-tree (sltx) and the cache as-well."),
        Arg('-e', '--exclude', action='append', metavar='pattern', dest='exclude_patterns',
            help="Exclude all files/directories matching this pattern. May be supplied multiple times.")
    ]),
    'auto-setup': ((cmd_auto_setup, []), Arg(description='Setup a basic version of sltx (this requires docker to be setup).'),
                   [
        Arg('-d', '--dependencies', dest='auto_deps', action='store_true',
            help='This will install the recommended dependencies on your host system. This is helpful if you have texlive installed and want your editor to recognize the libraries as well.')
    ]),
    'version': ((cmd_version, []), Arg(description='Show the version-info for sltx.'), [])
})

parser = argparse.ArgumentParser(
    description="sltx, a Simple LaTeX utility", epilog="sltx Version: " + su.get_version())

# commands, parser.add_mutually_exclusive_group()
parser.add_argument('-c', '--config', dest='config', metavar='config.yml',
                    required=False, type=valid_file,
                    help="the file to load the configuration from.")

parser.add_argument('-t', '--threads', metavar='N', dest='threads', type=int,
                    help="number of threads to run the installation. Default is 1. This number will only affect some routines.",
                    default=-1)

parser.add_argument('-q', '--quiet', dest='quiet',
                    required=False, action='store_true',
                    help="Set the flag if output is to be reduced")

parser.add_argument('-v', '--verbose', dest='verbose',
                    required=False, action='store_true',
                    help="Output in verbose mode -> more information from sltx")

parser.add_argument('--log', dest='log',
                    required=False, action='store_true',
                    help="Write to logfile. This is experimental.")

# TODO: Format support with mlatexformat?

cmd_parser = parser.add_subparsers(
    title='command', description="Select the command for sltx", metavar=set(sub_parser.cmds.keys()),
    help="Help for the specific command. You may use shortcuts for them: " +
    str([k[1] for k in sub_parser.helper_values() if len(k[1]) != 0]),
    dest='command')

sub_parser.generate(cmd_parser)
